@@ -8,7 +8,6 @@ import socket
 import struct
 import os
 import logging
-import signal
 import json
 from collections import defaultdict
 from time import time
@@ -130,25 +129,16 @@ class DHCPD:
             except ValueError:
                 pass
 
-        signal.signal(signal.SIGINT, self.export_leases)
-        signal.signal(signal.SIGTERM, self.export_leases)
-        signal.signal(signal.SIGALRM, self.export_leases)
-        signal.signal(signal.SIGHUP, self.export_leases)
-
-    def export_leases(self, signum, frame):
+    def export_leases(self):
         if self.save_leases_file:
             export_safe = dict()
             for lease in self.leases:
                 # translate the key to json safe (and human readable) mac
                 export_safe[self.get_mac(lease)] = self.leases[lease]
-            leases_file = open(self.save_leases_file, 'wb')
+            leases_file = open(self.save_leases_file, 'w')
             json.dump(export_safe, leases_file)
             self.logger.info('Exported leases to {0}'.format(self.save_leases_file))
 
-        # if keyboard interrupt, propagate upwards
-        if signum == signal.SIGINT:
-            raise KeyboardInterrupt
-
     def get_namespaced_static(self, path, fallback = {}):
         statics = self.static_config
         for child in path.split('.'):
@@ -248,7 +238,7 @@ class DHCPD:
         # BOOTP legacy pad
         response += b'\x00' * 64 # server name
         if self.mode_proxy:
-            response += self.file_name
+            response += self.file_name.encode('ascii')
             response += b'\x00' * (128 - len(self.file_name))
         else:
             response += b'\x00' * 128
@@ -300,7 +290,7 @@ class DHCPD:
 
         if self.mode_proxy:
             response += self.tlv_encode(60, 'PXEClient')
-            response += struct.pack('!BBBBBBB4sB', 43, 10, 6, 1, 0b1000, 10, 4, b'\x00' + 'PXE', 0xff)
+            response += struct.pack('!BBBBBBB4sB', 43, 10, 6, 1, 0b1000, 10, 4, b'\x00' + b'PXE', 0xff)
         response += b'\xff'
         return response
 
