@@ -6,6 +6,7 @@ import sys
 import json
 import logging
 import logging.handlers
+import signal
 
 try:
     import argparse
@@ -38,6 +39,7 @@ SETTINGS = {'NETBOOT_DIR':'netboot',
             'STATIC_CONFIG':'',
             'SYSLOG_SERVER':None,
             'SYSLOG_PORT':514,
+            'TFTP_PORT':69,
             'TFTP_SERVER_IP':'0.0.0.0',
             'USE_IPXE':False,
             'USE_HTTP':False,
@@ -118,6 +120,7 @@ def parse_cli_arguments():
 
     # TFTP server arguments
     tftp_group = parser.add_argument_group(title = 'TFTP', description = 'Arguments relevant to the TFTP server')
+    tftp_group.add_argument('--tftp-port', action = 'store', dest = 'TFTP_PORT', help = 'TFTP Server Port', default = SETTINGS['TFTP_PORT'])
     tftp_group.add_argument('--tftp-server-ip', action = 'store', dest = 'TFTP_SERVER_IP', help = 'TFTP Server IP', default = SETTINGS['TFTP_SERVER_IP'])
 
     return parser.parse_args()
@@ -161,7 +164,7 @@ def main():
             except ValueError:
                 sys.exit('{0} does not contain valid JSON'.format(args.JSON_CONFIG))
             for setting in loaded_config:
-                if type(loaded_config[setting]) is str:
+                if type(loaded_config[setting]) is bytes:
                     loaded_config[setting] = loaded_config[setting].encode('ascii')
             SETTINGS.update(loaded_config) # update settings with JSON config
             args = parse_cli_arguments() # re-parse, CLI options take precedence
@@ -170,22 +173,6 @@ def main():
         if os.getuid() != 0:
             print(sys.stderr, '\nWARNING: Not root. Servers will probably fail to bind.\n')
 
-
-        # ideally this would be in dhcp itself, but the chroot below *probably*
-        # breaks the ability to open the config file.
-        if args.STATIC_CONFIG:
-            try:
-                static_config = io.open(args.STATIC_CONFIG, 'r')
-            except IOError:
-                sys.exit("Failed to open {0}".format(args.STATIC_CONFIG))
-            try:
-                loaded_statics = json.load(static_config)
-                static_config.close()
-            except ValueError:
-                sys.exit("{0} does not contain valid json".format(args.STATIC_CONFIG))
-        else:
-            loaded_statics = dict()
-
         # setup main logger
         sys_logger = logging.getLogger('PyPXE')
         if args.SYSLOG_SERVER:
@@ -201,7 +188,7 @@ def main():
         if args.USE_HTTP and not args.USE_IPXE and not args.USE_DHCP:
             sys_logger.warning('HTTP selected but iPXE disabled. PXE ROM must support HTTP requests.')
 
-        # if the argument was pased to enabled ProxyDHCP then enable the DHCP server
+        # if the argument was passed to enabled ProxyDHCP then enable the DHCP server
         if args.DHCP_MODE_PROXY:
             args.USE_DHCP = True
 
@@ -241,6 +228,7 @@ def main():
                 mode_verbose = do_verbose('tftp'),
                 logger = tftp_logger,
                 netboot_directory = args.NETBOOT_DIR,
+                port = args.TFTP_PORT,
                 ip = args.TFTP_SERVER_IP)
             tftpd = threading.Thread(target = tftp_server.listen)
             tftpd.daemon = True
@@ -250,6 +238,38 @@ def main():
         # configure/start DHCP server
         if args.USE_DHCP:
 
+            def dhcp_export_leases(signum, frame):
+                dhcp_server.export_leases()
+
+                # if keyboard interrupt, propagate upwards
+                if signum == signal.SIGINT:
+                    raise KeyboardInterrupt
+
+            def load_dhcp_static_config(filename):
+                result = ['', dict()]
+
+                try:
+                    static_config = io.open(filename, 'r')
+                except IOError:
+                    result[0] = "Failed to open {0}".format(filename)
+                    return result
+
+                try:
+                    result[1] = json.load(static_config)
+                    static_config.close()
+                except ValueError:
+                    result[0] = "{0} does not contain valid json".format(filename)
+                    return result
+
+                return result
+
+            if args.STATIC_CONFIG:
+                message, loaded_statics = load_dhcp_static_config(args.STATIC_CONFIG)
+                if message:
+                    sys.exit(message)
+            else:
+                loaded_statics = dict()
+
             # setup DHCP logger
             dhcp_logger = helpers.get_child_logger(sys_logger, 'DHCP')
             if args.DHCP_MODE_PROXY:
@@ -278,6 +298,10 @@ def main():
                 static_config = loaded_statics,
                 logger = dhcp_logger,
                 saveleases = args.LEASES_FILE)
+            signal.signal(signal.SIGINT, dhcp_export_leases)
+            signal.signal(signal.SIGTERM, dhcp_export_leases)
+            signal.signal(signal.SIGALRM, dhcp_export_leases)
+            signal.signal(signal.SIGHUP, dhcp_export_leases)
             dhcpd = threading.Thread(target = dhcp_server.listen)
             dhcpd.daemon = True
             dhcpd.start()
